-------------------------------------------------------------------------------
-- 16-bit instruction register
--
-- Ports:
--   - clk [in]  : clock signal
--   - ce  [in]  : clock enable signal
--   - rst [in]  : reset signal
--
--   - instr [in]  : instruction word from memory
--   - cond  [out] : condition
--   - op    [out] : opcode
--   - updt  [out] : update flag
--   - valx  [out] : register rX or window address @Y
--   - imm   [out] : immediate flag
--   - valy  [out] : register rY, value Y or window address @X
--
--         __ __ __ __ __ __ __ __ __ __ __ __ __ __ __ __
-- Bit #  |15|14|13|12|11|10| 9| 8| 7| 6| 5| 4| 3| 2| 1| 0|
--        |__|__|__|__|__|__|__|__|__|__|__|__|__|__|__|__|
--
--        |--------|-----------|--|--------|--|-----------|
--        Condition   Opcode    |  rX / @Y  |  rY / Y / @X
--                      Update -'           '- Immediate
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity instr_reg is
  port ( clk : in  std_logic;
         ce  : in  std_logic;
         rst : in  std_logic;

         instr : in  std_logic_vector(15 downto 0);
         cond  : out std_logic_vector(2 downto 0);
         op    : out std_logic_vector(3 downto 0);
         updt  : out std_logic;
         valx  : out std_logic_vector(2 downto 0);
         imm   : out std_logic;
         valy  : out std_logic_vector(3 downto 0) );
end entity;

architecture arch of instr_reg is
begin
end architecture;

-------------------------------------------------------------------------------
-- 4-bit status register
--
-- Ports:
--   - clk [in]  : clock signal
--   - ce  [in]  : clock enable signal
--   - rst [in]  : reset signal
--
--   - i [in]  : status flags from ALU
--   - o [out] : latched status flags
--
--         __ __ __ __
-- Bit #  | 3| 2| 1| 0|
--        |__|__|__|__|
--          Z  N  C  V
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity status_reg is
  port ( clk : in  std_logic;
         ce  : in  std_logic;
         rst : in  std_logic;

         i : in  std_logic_vector(3 downto 0);
         o : out std_logic_vector(3 downto 0) );
end entity;

architecture arch of status_reg is
begin
end architecture;

-------------------------------------------------------------------------------
-- 64 x 16 bits windowed register file
--
-- Ports:
--   - clk [in]  : clock signal
--   - rst [in]  : reset signal
--
--   - valx [in] : register rX or window address @Y
--   - valy [in] : register rY or window address @X
--
--   - rx_out [out] : rX output value
--   - ry_out [out] : rY output value
--   - din    [in]  : input value
--
--   - rx_ce  [in] : clock enable signal for destination register rX
--   - win_ce [in] : clock enable signal for window registers wX and wY
--
-- Mapping between virtual registers (rX and rY) and physical registers (pX and
-- pY) using window register (wX and wY):
--
--     rX = p(wX*4 + X)
--     rY = p(wY*8 + Y)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity reg_file is
  port ( clk : in  std_logic;
         rst : in  std_logic;

         valx : in std_logic_vector(2 downto 0);
         valy : in std_logic_vector(3 downto 0);

         rx_out : out std_logic_vector(15 downto 0);
         ry_out : out std_logic_vector(15 downto 0);
         din    : in  std_logic_vector(15 downto 0);

         rx_ce  : in  std_logic;
         win_ce : in  std_logic );
end entity;

architecture arch of reg_file is
begin
end architecture;
