library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity instr_reg_test is
end entity;

architecture arch of instr_reg_test is
  signal done   : boolean := false;
  signal passed : boolean := true;
  signal ok     : boolean := true;

  component instr_reg is
    port ( clk : in  std_logic;
           ce  : in  std_logic;
           rst : in  std_logic;

           instr : in  std_logic_vector(15 downto 0);
           cond  : out std_logic_vector(2 downto 0);
           op    : out std_logic_vector(3 downto 0);
           updt  : out std_logic;
           valx  : out std_logic_vector(2 downto 0);
           imm   : out std_logic;
           valy  : out std_logic_vector(3 downto 0) );
  end component;

  signal clk : std_logic;
  signal ce  : std_logic;
  signal rst : std_logic;
  
  signal instr : std_logic_vector(15 downto 0);
  signal cond  : std_logic_vector(2 downto 0);
  signal op    : std_logic_vector(3 downto 0);
  signal updt  : std_logic;
  signal valx  : std_logic_vector(2 downto 0);
  signal imm   : std_logic;
  signal valy  : std_logic_vector(3 downto 0);
begin

  instr_reg_0 : instr_reg
    port map ( clk => clk,
               ce  => ce,
               rst => rst,

               instr => instr,
               cond  => cond,
               op    => op,
               updt  => updt,
               valx  => valx,
               imm   => imm,
               valy  => valy );

  -----------------------------------------------------------------------------
  -- Clock signal
  -----------------------------------------------------------------------------

  process
  begin
    if not done then
      clk <= '1';
      wait for 5 ns;
      clk <= '0';
      wait for 5 ns;
    else
      clk <= 'U';
      wait;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- Input stimuli
  -----------------------------------------------------------------------------

  process
  begin
    -- Reset ------------------------------------------------------------- 0 ns
    ce    <= '1';
    rst   <= '1';
    instr <= X"FFFF";
    wait for 12 ns;
    -- Clock enabled ---------------------------------------------------- 12 ns
    ce    <= '1';
    rst   <= '0';
    instr <= X"5AA5";
    wait for 10 ns;
    -- Clock disabled --------------------------------------------------- 22 ns
    ce    <= '0';
    rst   <= '0';
    instr <= X"FFFF";
    wait for 10 ns;
    --------------------------------------------------------------------- 32 ns
    ce    <= 'U';
    rst   <= 'U';
    instr <= "UUUUUUUUUUUUUUUU";
    wait;
  end process;

  -----------------------------------------------------------------------------
  -- Output verification
  -----------------------------------------------------------------------------

  process
  begin
    wait for 9 ns;
    -- Reset (before clk) ------------------------------------------------ 9 ns
    ok <= (cond = "000" ) and
          (op   = "0000") and
          (updt = '0'   ) and
          (valx = "000" ) and
          (imm  = '0'   ) and
          (valy = "0000");
    wait for 2 ns;
    -- Reset (after clk) ------------------------------------------------ 11 ns
    ok <= (cond = "000" ) and
          (op   = "0000") and
          (updt = '0'   ) and
          (valx = "000" ) and
          (imm  = '0'   ) and
          (valy = "0000");
    wait for 8 ns;
    -- Clock enabled (before clk) --------------------------------------- 19 ns
    ok <= (cond = "000" ) and
          (op   = "0000") and
          (updt = '0'   ) and
          (valx = "000" ) and
          (imm  = '0'   ) and
          (valy = "0000");
    wait for 2 ns;
    -- Clock enabled (after clk) ---------------------------------------- 21 ns
    ok <= (cond = "010" ) and
          (op   = "1101") and
          (updt = '0'   ) and
          (valx = "101" ) and
          (imm  = '0'   ) and
          (valy = "0101");
    wait for 8 ns;
    -- Clock disabled (before clk) -------------------------------------- 29 ns
    ok <= (cond = "010" ) and
          (op   = "1101") and
          (updt = '0'   ) and
          (valx = "101" ) and
          (imm  = '0'   ) and
          (valy = "0101");
    wait for 2 ns;
    -- Clock disabled (after clk) --------------------------------------- 31 ns
    ok <= (cond = "010" ) and
          (op   = "1101") and
          (updt = '0'   ) and
          (valx = "101" ) and
          (imm  = '0'   ) and
          (valy = "0101");
    wait for 1 ns;
    --------------------------------------------------------------------- 32 ns
    ok   <= true;
    done <= true;
    wait;
  end process;

  passed <= passed and ok;

end architecture;

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity status_reg_test is
end entity;

architecture arch of status_reg_test is
  signal done   : boolean := false;
  signal passed : boolean := true;
  signal ok     : boolean := true;

  component status_reg is
   port ( clk : in  std_logic;
          ce  : in  std_logic;
          rst : in  std_logic;

          i : in  std_logic_vector(3 downto 0);
          o : out std_logic_vector(3 downto 0) );
  end component;

  signal clk : std_logic;
  signal ce  : std_logic;
  signal rst : std_logic;
  
  signal i : std_logic_vector(3 downto 0);
  signal o : std_logic_vector(3 downto 0);
begin

  status_reg_0 : status_reg
    port map ( clk => clk,
               ce  => ce,
               rst => rst,

               i => i,
               o => o );

  -----------------------------------------------------------------------------
  -- Clock signal
  -----------------------------------------------------------------------------

  process
  begin
    if not done then
      clk <= '1';
      wait for 5 ns;
      clk <= '0';
      wait for 5 ns;
    else
      clk <= 'U';
      wait;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- Input stimuli
  -----------------------------------------------------------------------------

  process
  begin
    -- Reset ------------------------------------------------------------- 0 ns
    ce  <= '0';
    rst <= '1';
    i   <= "1111";
    wait for 12 ns;
    -- Clock enabled ---------------------------------------------------- 12 ns
    ce  <= '1';
    rst <= '0';
    i   <= "1010";
    wait for 10 ns;
    -- Clock disabled --------------------------------------------------- 22 ns
    ce  <= '0';
    rst <= '0';
    i   <= "1111";
    wait for 10 ns;
    --------------------------------------------------------------------- 32 ns
    ce  <= 'U';
    rst <= 'U';
    i   <= "UUUU";
    wait;
  end process;

  -----------------------------------------------------------------------------
  -- Output verification
  -----------------------------------------------------------------------------

  process
  begin
    wait for 9 ns;
    -- Reset (before clk) ------------------------------------------------ 9 ns
    ok <= (o = "0000");
    wait for 2 ns;
    -- Reset (after clk) ------------------------------------------------ 11 ns
    ok <= (o = "0000");
    wait for 8 ns;
    -- Clock enabled (before clk) --------------------------------------- 19 ns
    ok <= (o = "0000");
    wait for 2 ns;
    -- Clock enabled (after clk) ---------------------------------------- 21 ns
    ok <= (o = "1010");
    wait for 8 ns;
    -- Clock disabled (before clk) -------------------------------------- 29 ns
    ok <= (o = "1010");
    wait for 2 ns;
    -- Clock disabled (after clk) --------------------------------------- 31 ns
    ok <= (o = "1010");
    wait for 1 ns;
    --------------------------------------------------------------------- 32 ns
    ok   <= true;
    done <= true;
    wait;
  end process;

  passed <= passed and ok;

end architecture;

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity reg_file_test is
end entity;

architecture arch of reg_file_test is
  signal done   : boolean := false;
  signal passed : boolean := true;
  signal ok     : boolean := true;

  component reg_file is
    port ( clk : in  std_logic;
           rst : in  std_logic;

           valx : in std_logic_vector(2 downto 0);
           valy : in std_logic_vector(3 downto 0);

           rx_out : out std_logic_vector(15 downto 0);
           ry_out : out std_logic_vector(15 downto 0);
           din    : in  std_logic_vector(15 downto 0);

           rx_ce  : in  std_logic;
           win_ce : in  std_logic );
  end component;

  signal clk : std_logic;
  signal rst : std_logic;
  
  signal valx : std_logic_vector(2 downto 0);
  signal valy : std_logic_vector(3 downto 0);

  signal rx_out : std_logic_vector(15 downto 0);
  signal ry_out : std_logic_vector(15 downto 0);
  signal din    : std_logic_vector(15 downto 0);

  signal win_ce : std_logic;
  signal rx_ce  : std_logic;
begin

  reg_file_0 : reg_file
    port map ( clk => clk,
               rst => rst,

               valx => valx,
               valy => valy,

               rx_out => rx_out,
               ry_out => ry_out,
               din    => din,

               rx_ce  => rx_ce,
               win_ce => win_ce );

  -----------------------------------------------------------------------------
  -- Clock signal
  -----------------------------------------------------------------------------

  process
  begin
    if not done then
      clk <= '1';
      wait for 5 ns;
      clk <= '0';
      wait for 5 ns;
    else
      clk <= 'U';
      wait;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- Input stimuli
  -----------------------------------------------------------------------------

  process
  begin
    -- Reset ------------------------------------------------------------- 0 ns
    rst    <= '1';
    rx_ce  <= '0';
    win_ce <= '0';
    valx   <= "000";
    valy   <= "0000";
    din    <= X"FFFF";
    wait for 2 ns;
    -- Write all regs ---------------------------------------------------------
    for i in 0 to 7 loop
      wait for 10 ns;
      -- Set wX window on R(8i) ---------------------------------- 12 + 90i ns
      rst    <= '0';
      rx_ce  <= '0';
      win_ce <= '1';
      valx   <= conv_std_logic_vector((i+1) mod 8, 3);
      valy   <= conv_std_logic_vector(2*i, 4);
      din    <= X"FFFF";
      -- Write regs in window -------------------------------------------------
      for j in 0 to 7 loop
        wait for 10 ns;
        -- Write reg R(8i+j) ------------------------------ 22 + 90i + 10j ns
        rst    <= '0';
        rx_ce  <= '1';
        win_ce <= '0';
        valx   <= conv_std_logic_vector(j, 3);
        valy   <= conv_std_logic_vector(j+8, 4);
        din    <= (not conv_std_logic_vector(8*i+j, 8)) &
                       conv_std_logic_vector(8*i+j, 8);
      end loop;
    end loop;
    -- Check all regs ---------------------------------------------------------
    for i in 0 to 15 loop
      wait for 10 ns;
      -- Set wX to R(4i) and wY to R(8(i/2)) ------------------- 732 + 170i ns
      rst    <= '0';
      rx_ce  <= '0';
      win_ce <= '1';
      valx   <= conv_std_logic_vector(i/2, 3);
      valy   <= conv_std_logic_vector(i, 4);
      din    <= X"FFFF";
      -- Check regs in window -------------------------------------------------
      for j in 0 to 15 loop
        wait for 10 ns;
        -- Check regs R(4i+j/2) and R(8(i/2)+j) --------- 742 + 170i + 10j ns
        rst    <= '0';
        rx_ce  <= '0';
        win_ce <= '0';
        valx   <= conv_std_logic_vector(j/2, 3);
        valy   <= conv_std_logic_vector(j, 4);
        din    <= X"FFFF";
      end loop;
    end loop;
    wait for 10 ns;
    ------------------------------------------------------------------- 3452 ns
    rst    <= 'U';
    rx_ce  <= 'U';
    win_ce <= 'U';
    valx   <= "UUU";
    valy   <= "UUUU";
    din    <= "UUUUUUUUUUUUUUUU";
    wait;
  end process;

  -----------------------------------------------------------------------------
  -- Output verification
  -----------------------------------------------------------------------------

  process
  begin
    wait for 9 ns;
    -- Reset (before clk) ------------------------------------------------ 9 ns
    ok <= (rx_out = X"0000") and
          (ry_out = X"0000");
    wait for 2 ns;
    -- Reset (after clk) ------------------------------------------------ 11 ns
    ok <= (rx_out = X"0000") and
          (ry_out = X"0000");
    -- Write all regs ---------------------------------------------------------
    for i in 0 to 7 loop
      wait for 8 ns;
      -- Set wX window on R(8i) (before clk) --------------------- 19 + 90i ns
      if i = 0 then
        ok <= (rx_out = X"0000") and
              (ry_out = X"0000");
      elsif i < 7 then
        ok <= (rx_out = (not conv_std_logic_vector(8*(i-1) + (i+1) mod 8, 8)) &
                             conv_std_logic_vector(8*(i-1) + (i+1) mod 8, 8)   ) and
              (ry_out = X"0000");
      else
        ok <= (rx_out = (not conv_std_logic_vector(8*(i-1) + (i+1) mod 8, 8)) &
                             conv_std_logic_vector(8*(i-1) + (i+1) mod 8, 8)   ) and
              (ry_out = (not conv_std_logic_vector((8*i+2*i) mod 64, 8)) &
                             conv_std_logic_vector((8*i+2*i) mod 64, 8)   );
      end if;
      wait for 2 ns;
      -- Set wX window on R(8i) (after clk) ---------------------- 21 + 90i ns
      if i < 6 then
        ok <= (rx_out = X"0000") and
              (ry_out = X"0000");
      else
        ok <= (rx_out = X"0000") and
              (ry_out = (not conv_std_logic_vector((8*(i+1)+2*i) mod 64, 8)) &
                             conv_std_logic_vector((8*(i+1)+2*i) mod 64, 8)   );
      end if;
      -- Write regs in window -------------------------------------------------
      for j in 0 to 7 loop
        wait for 8 ns;
        -- Write reg R(8i+j) (before clk) ----------------- 29 + 90i + 10j ns
        if i < 6 then
          ok <= (rx_out = X"0000") and
                (ry_out = X"0000");
        else
          ok <= (rx_out = X"0000") and
                (ry_out = (not conv_std_logic_vector((8*(i+2)+j) mod 64, 8)) &
                               conv_std_logic_vector((8*(i+2)+j) mod 64, 8)   );
        end if;
        wait for 2 ns;
        -- Write reg R(8i+j) (after clk) ------------------ 31 + 90i + 10j ns
        if i < 6 then
          ok <= (rx_out = (not conv_std_logic_vector(8*i+j, 8)) &
                               conv_std_logic_vector(8*i+j, 8)   ) and
                (ry_out = X"0000");
        else
          ok <= (rx_out = (not conv_std_logic_vector(8*i+j, 8)) &
                               conv_std_logic_vector(8*i+j, 8)   ) and
                (ry_out = (not conv_std_logic_vector((8*(i+2)+j) mod 64, 8)) &
                               conv_std_logic_vector((8*(i+2)+j) mod 64, 8)   );
        end if;
      end loop;
    end loop;
    -- Check all regs ---------------------------------------------------------
    for i in 0 to 15 loop
      wait for 8 ns;
      -- Set wX to R(4i) and wY to R(8(i/2)) (before clk) ------ 739 + 170i ns
      if i = 0 then
        ok <= (rx_out = (not conv_std_logic_vector(8*7+0, 8)) &
                             conv_std_logic_vector(8*7+0, 8)   ) and
              (ry_out = (not conv_std_logic_vector(8*0+0, 8)) &
                             conv_std_logic_vector(8*0+0, 8)   );
      else
        ok <= (rx_out = (not conv_std_logic_vector((4*(i-1)+(i/2)) mod 64, 8)) &
                             conv_std_logic_vector((4*(i-1)+(i/2)) mod 64, 8)   ) and
              (ry_out = (not conv_std_logic_vector((8*((i-1)/2)+i) mod 64, 8)) &
                             conv_std_logic_vector((8*((i-1)/2)+i) mod 64, 8)   );
      end if;
      wait for 2 ns;
      -- Set wX to R(4i) and wY to R(8(i/2)) (after clk) ------- 741 + 170i ns
      ok <= (rx_out = (not conv_std_logic_vector((4*i+(i/2)) mod 64, 8)) &
                           conv_std_logic_vector((4*i+(i/2)) mod 64, 8)   ) and
            (ry_out = (not conv_std_logic_vector((8*(i/2)+i) mod 64, 8)) &
                           conv_std_logic_vector((8*(i/2)+i) mod 64, 8)   );
      -- Check regs in window -------------------------------------------------
      for j in 0 to 15 loop
        wait for 8 ns;
        -- Check regs R(4i+j/2) and R(8(i/2)+j) (before clk) ------------------
        ------------------------------------------------- 749 + 170i + 10j ns
        ok <= (rx_out = (not conv_std_logic_vector((4*i+(j/2)) mod 64, 8)) &
                             conv_std_logic_vector((4*i+(j/2)) mod 64, 8)   ) and
              (ry_out = (not conv_std_logic_vector((8*(i/2)+j) mod 64, 8)) &
                             conv_std_logic_vector((8*(i/2)+j) mod 64, 8)   );
        wait for 2 ns;
        -- Check regs R(4i+j/2) and R(8(i/2)+j) (after clk) -------------------
        ------------------------------------------------- 751 + 170i + 10j ns
        ok <= (rx_out = (not conv_std_logic_vector((4*i+(j/2)) mod 64, 8)) &
                             conv_std_logic_vector((4*i+(j/2)) mod 64, 8)   ) and
              (ry_out = (not conv_std_logic_vector((8*(i/2)+j) mod 64, 8)) &
                             conv_std_logic_vector((8*(i/2)+j) mod 64, 8)   );
      end loop;
    end loop;
    wait for 1 ns;
    ------------------------------------------------------------------- 3252 ns
    ok   <= true;
    done <= true;
    wait;
  end process;

  passed <= passed and ok;

end architecture;
